from django import forms
from edc_constants.constants import NO, NOT_APPLICABLE, PENDING, YES
from edc_dx_review.utils import is_rx_initiated, raise_if_clinical_review_does_not_exist
from intecomm_form_validators.subject import HivReviewFormValidator

from ..models import HivReview
from .mixins import CrfModelFormMixin


class HivReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HivReviewFormValidator

    def clean(self):
        cleaned_data = super().clean()
        raise_if_clinical_review_does_not_exist(cleaned_data.get("subject_visit"))
        if is_rx_initiated(
            subject_identifier=self.get_subject_identifier(),
            report_datetime=self.report_datetime,
            instance_id=self.instance.id,
        ):
            if cleaned_data.get("rx_init") in [YES, NO]:
                raise forms.ValidationError(
                    {
                        "rx_init": (
                            "This field is applicable. Subject was NOT previously "
                            "reported as on ART."
                        )
                    }
                )

        elif cleaned_data.get("rx_init") == NOT_APPLICABLE:
            raise forms.ValidationError(
                {
                    "rx_init": (
                        "This field is applicable. Subject was NOT previously "
                        "reported as on ART."
                    )
                }
            )
        return cleaned_data

    def validate_viral_load(self):
        self.required_if(YES, PENDING, field="has_vl", field_required="drawn_date")
        if self.cleaned_data.get("drawn_date") and self.dx_date:
            if self.cleaned_data.get("drawn_date") < self.dx_date:
                raise forms.ValidationError(
                    {"drawn_date": "Invalid. Cannot be before HIV diagnosis."}
                )
        self.required_if(YES, field="has_vl", field_required="vl")
        self.required_if(YES, field="has_vl", field_required="vl_quantifier")

    class Meta:
        model = HivReview
        fields = "__all__"
        labels = {
            "has_vl": "Does the patient have a new or pending viral load result?",
            "vl": "Viral load result",
        }
        help_text = {"has_vl": "Do not report baseline results on this form."}



from django.db import models
from edc_model.models import BaseUuidModel
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow
from edc_visit_tracking.constants import SCHEDULED

from ..choices import MEETING_STATUS
from ..constants import ATTENDED
from .patient_group_appointment import PatientGroupAppointment


class PatientGroupMeeting(SiteModelMixin, BaseUuidModel):
    patient_group_appointment = models.OneToOneField(
        PatientGroupAppointment,
        on_delete=models.PROTECT,
        blank=False,
    )

    status = models.CharField(max_length=25, choices=MEETING_STATUS, default=SCHEDULED)

    report_datetime = models.DateTimeField(default=get_utcnow)

    meeting_datetime = models.DateTimeField(null=True, blank=True)

    patients = models.ManyToManyField("intecomm_screening.PatientLog", verbose_name="Patients")

    notes = models.TextField(null=True, blank=True)

    # duration,

    def __str__(self):
        return self.patient_group_appointment.patient_group.name

    def save(self, *args, **kwargs):
        self.status = ATTENDED if self.meeting_datetime else SCHEDULED
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Patient Group Meeting"
        verbose_name_plural = "Patient Groups Meeting"



#!/usr/bin/env python
from edc_test_settings.func_main import func_main2

if __name__ == "__main__":
    project_tests = [
        "intecomm_auth.tests",
        "intecomm_ae.tests",
        "intecomm_dashboard.tests",
        "intecomm_edc.tests",
        "intecomm_labs.tests",
        "intecomm_lists.tests",
        "intecomm_prn.tests",
        "intecomm_group.tests",
        "intecomm_screening.tests",
        "intecomm_subject.tests",
        "intecomm_consent.tests",
        "intecomm_visit_schedule.tests",
        "intecomm_form_validators.tests",
        "intecomm_rando.tests",
        "intecomm_eligibility.tests",
    ]

    func_main2("tests.test_settings", *project_tests)



# Generated by Django 4.1.2 on 2022-11-21 22:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0005_historicalpatientcall_alt_contact_number_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalpatientcall",
            name="willing_to_attend",
        ),
        migrations.RemoveField(
            model_name="patientcall",
            name="willing_to_attend",
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="catchment_area",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_sure", "Not sure"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Does the participant still reside within the catchment area",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_sure", "Not sure"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Is the participant known to be alive",
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="catchment_area",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_sure", "Not sure"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Does the participant still reside within the catchment area",
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_sure", "Not sure"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                verbose_name="Is the participant known to be alive",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientcall",
            name="attend_date",
            field=models.DateField(
                blank=True, null=True, verbose_name="When can the patient next attend"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientcall",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="May we call again?",
            ),
        ),
        migrations.AlterField(
            model_name="patientcall",
            name="attend_date",
            field=models.DateField(
                blank=True, null=True, verbose_name="When can the patient next attend"
            ),
        ),
        migrations.AlterField(
            model_name="patientcall",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="May we call again?",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-04-20 15:58

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0034_clinicalreviewbaseline_protocol_incident_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="drugrefilldm",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefilldm",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhiv",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhtn",
            name="return_in_days",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(180),
                ],
                verbose_name="In how many days has the patient been asked to return to clinic for a drug refill?",
            ),
        ),
    ]



import os
import sys

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management.color import color_style

style = color_style()


def update_forms_reference(sender=None, **kwargs):
    from edc_form_describer import FormsReference

    from intecomm_subject.admin_site import intecomm_subject_admin
    from intecomm_visit_schedule.visit_schedules import visit_schedule

    sys.stdout.write(
        style.MIGRATE_HEADING("Refreshing CRF reference document for intecomm_subject\n")
    )
    doc_folder = os.path.join(settings.BASE_DIR, "docs")
    if not os.path.exists(doc_folder):
        os.mkdir(doc_folder)
    forms = FormsReference(
        visit_schedules=[visit_schedule],
        admin_site=intecomm_subject_admin,
        title="INTECOMM-EDC Forms Reference",
        add_per_form_timestamp=False,
    )
    path = os.path.join(doc_folder, "forms_reference.md")
    forms.to_file(path=path, overwrite=True)


class Command(BaseCommand):
    help = "Update forms reference document (.md)"

    def handle(self, *args, **options):
        update_forms_reference()



# Generated by Django 4.2.3 on 2023-07-26 18:24

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0054_alter_healthtalklog_unique_together"),
    ]

    operations = [
        migrations.DeleteModel(
            name="IdentifierFormat",
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-05 23:20
from django.db import migrations
from edc_list_data import PreloadData
from tqdm import tqdm

from intecomm_lists.list_data import list_data


def update_infosources(apps, schema_editor):
    PreloadData(list_data=list_data, apps=apps)
    old_model_cls = apps.get_model("edc_next_appointment.infosources")
    new_model_cls = apps.get_model("edc_appointment.infosources")
    crf_model_cls = apps.get_model("intecomm_subject.nextappointment")
    total = crf_model_cls.objects.all().count()
    for obj in tqdm(crf_model_cls.objects.all(), total=total):
        info_source = old_model_cls.objects.get(id=obj.info_source_id)
        info_source_new = new_model_cls.objects.get(name=info_source.name)
        obj.info_source_new = info_source_new
        obj.save_base(update_fields=["info_source_new"])


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0118_historicalnextappointment_info_source_new_and_more"),
    ]

    operations = [migrations.RunPython(update_infosources)]



from dataclasses import dataclass, field
from datetime import date

import numpy as np
import pandas as pd
from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import YES
from tqdm import tqdm

from intecomm_prn.models import OffScheduleComm, OffScheduleInte
from intecomm_subject.models import HivInitialReview, HivReview

from ..models import Diagnoses as DiagnosesModel


@dataclass
class Vl:
    baseline_vl: int
    baseline_vl_date: date
    endline_vl: int
    endline_vl_date: date


@dataclass
class Data:
    subject_identifier: str
    baseline_date: date
    dx_date: date
    baseline_vl: int | None = field(default=None)
    baseline_vl_date: date | None = field(default=None)
    endline_vl: int | None = field(default=None)
    endline_vl_date: date | None = field(default=None)

    @property
    def offschedule_date(self) -> date:
        offschedule_date = None
        try:
            obj = OffScheduleComm.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            try:
                obj = OffScheduleInte.objects.get(subject_identifier=self.subject_identifier)
            except ObjectDoesNotExist:
                pass
            else:
                offschedule_date = obj.offschedule_datetime.date()
        else:
            offschedule_date = obj.offschedule_datetime.date()
        return offschedule_date

    def as_tuple(self):
        return (
            self.subject_identifier,
            self.baseline_date,
            self.dx_date,
            self.baseline_vl,
            self.baseline_vl_date,
            self.endline_vl,
            self.endline_vl_date,
            self.offschedule_date,
        )


def get_vl_columns(review_obj: HivInitialReview | HivReview, baseline_date: date) -> Vl:
    drawn_date = review_obj.drawn_date or review_obj.report_datetime.date()
    baseline_vl = (
        review_obj.vl if drawn_date <= baseline_date + relativedelta(months=3) else np.nan
    )
    baseline_vl_date = drawn_date if drawn_date <= baseline_date else pd.NaT
    endline_vl = (
        review_obj.vl if drawn_date >= baseline_date + relativedelta(months=6) else np.nan
    )
    endline_vl_date = (
        drawn_date if drawn_date >= baseline_date + relativedelta(months=6) else pd.NaT
    )
    return Vl(baseline_vl, baseline_vl_date, endline_vl, endline_vl_date)


def get_vl_data(path=None) -> pd.DataFrame:
    qs = DiagnosesModel.objects.filter(hiv=1).order_by("subject_identifier")
    total = qs.count()
    all_data = []
    for diagnoses in tqdm(qs, total=total):
        data = Data(
            diagnoses.subject_identifier,
            diagnoses.baseline_date,
            diagnoses.hiv_dx_date,
        )
        try:
            hiv_initial_review = HivInitialReview.objects.get(
                subject_visit__subject_identifier=diagnoses.subject_identifier
            )
        except ObjectDoesNotExist:
            pass
        else:
            if hiv_initial_review.has_vl == YES:
                vl = get_vl_columns(hiv_initial_review, diagnoses.baseline_date)
                data.dx_date = diagnoses.hiv_dx_date
                data.baseline_vl = vl.baseline_vl
                data.baseline_vl_date = vl.baseline_vl_date
        for hiv_review in HivReview.objects.filter(
            subject_visit__subject_identifier=diagnoses.subject_identifier
        ):
            if hiv_review.has_vl == YES:
                vl = get_vl_columns(hiv_review, diagnoses.baseline_date)
                if not data.baseline_vl:
                    data.baseline_vl = vl.baseline_vl
                    data.baseline_vl_date = vl.baseline_vl_date
                if not data.endline_vl:
                    data.endline_vl = vl.endline_vl
                    data.endline_vl_date = vl.endline_vl_date
        all_data.append(data.as_tuple())
    np_data = np.array(all_data)
    df = pd.DataFrame(
        np_data,
        columns=[
            "subject_identifier",
            "baseline_date",
            "dx_date",
            "baseline_vl_value",
            "baseline_vl_date",
            "endline_vl_value",
            "endline_vl_date",
            "offschedule_date",
        ],
    )
    if path:
        df.to_csv(path_or_buf=path, index=False)
    return df



# Generated by Django 4.2.3 on 2023-07-26 13:14

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        # ("intecomm_facility", "0002_alter_healthfacility_device_created_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("intecomm_lists", "0006_consentrefusalreasons_screeningrefusalreasons_and_more"),
        ("intecomm_screening", "0046_alter_historicalpatientlog_legal_name_and_more"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="healthtalklog",
            unique_together=set(),
        ),
        migrations.RenameField(
            model_name="historicalhealthtalklog",
            old_name="health_facility",
            new_name="old_health_facility",
        ),
        migrations.RenameField(
            model_name="healthtalklog",
            old_name="health_facility",
            new_name="old_health_facility",
        ),
        migrations.RenameField(
            model_name="idenfifierformat",
            old_name="health_facility",
            new_name="old_health_facility",
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-26 15:07

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0012_remove_vlsummary2_site_alter_diagnoses_managers_and_more"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "with visits as (\n                select subject_identifier, report_datetime, visit_code, visit_code_sequence\n                from intecomm_subject_subjectvisit\n                group by subject_identifier, report_datetime, visit_code, visit_code_sequence\n                order by subject_identifier, report_datetime, visit_code, visit_code_sequence\n            )\n     select *, uuid() as id, now() as `created`, 'intecomm_reports.subjectstransferred_view' as `report_model` from (SELECT prn.subject_identifier, prn.site_id, consent_datetime AS consented, v.visit_code, v.report_datetime AS last_visit, prn.report_datetime AS transferred, DATEDIFF(prn.report_datetime, consent_datetime) / 30 AS months, eof.offstudy_datetime AS offstudy, eof.report_datetime AS last_seen FROM intecomm_prn_subjecttransfer AS prn LEFT JOIN edc_registration_registeredsubject AS rs ON prn.subject_identifier = rs.subject_identifier LEFT JOIN intecomm_prn_endofstudy AS eof ON prn.subject_identifier = eof.subject_identifier LEFT JOIN (SELECT DISTINCT subject_identifier, LAST_VALUE(report_datetime) OVER w AS report_datetime, LAST_VALUE(visit_code) OVER w AS visit_code FROM visits WINDOW w AS (PARTITION BY `subject_identifier`)) AS v ON prn.subject_identifier = v.subject_identifier ORDER BY DATEDIFF(prn.report_datetime, consent_datetime) / 30 DESC, subject_identifier) as A ORDER BY site_id, subject_identifier",
                "subjects_transferred_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "subjects_transferred_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "with visits as (\n                select subject_identifier, report_datetime, visit_code, visit_code_sequence\n                from intecomm_subject_subjectvisit\n                group by subject_identifier, report_datetime, visit_code, visit_code_sequence\n                order by subject_identifier, report_datetime, visit_code, visit_code_sequence\n            )\n     select *, get_random_uuid() as id, now() as created, 'intecomm_reports.subjectstransferred_view' as report_model from (SELECT prn.subject_identifier, prn.site_id, consent_datetime AS consented, v.visit_code, v.report_datetime AS last_visit, prn.report_datetime AS transferred, CAST(CAST(AGE(CAST(prn.report_datetime AS TIMESTAMP), CAST(consent_datetime AS TIMESTAMP)) AS BIGINT) AS DOUBLE PRECISION) / NULLIF(30, 0) AS months, eof.offstudy_datetime AS offstudy, eof.report_datetime AS last_seen FROM intecomm_prn_subjecttransfer AS prn LEFT JOIN edc_registration_registeredsubject AS rs ON prn.subject_identifier = rs.subject_identifier LEFT JOIN intecomm_prn_endofstudy AS eof ON prn.subject_identifier = eof.subject_identifier LEFT JOIN (SELECT DISTINCT subject_identifier, LAST_VALUE(report_datetime) OVER w AS report_datetime, LAST_VALUE(visit_code) OVER w AS visit_code FROM visits WINDOW w AS (PARTITION BY \"subject_identifier\")) AS v ON prn.subject_identifier = v.subject_identifier ORDER BY CAST(CAST(AGE(CAST(prn.report_datetime AS TIMESTAMP), CAST(consent_datetime AS TIMESTAMP)) AS BIGINT) AS DOUBLE PRECISION) / NULLIF(30, 0) DESC NULLS LAST, subject_identifier NULLS FIRST) as A ORDER BY site_id, subject_identifier",
                "subjects_transferred_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "subjects_transferred_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "with visits as (\n                select subject_identifier, report_datetime, visit_code, visit_code_sequence\n                from intecomm_subject_subjectvisit\n                group by subject_identifier, report_datetime, visit_code, visit_code_sequence\n                order by subject_identifier, report_datetime, visit_code, visit_code_sequence\n            )\n     select *, uuid() as id, datetime() as created, 'intecomm_reports.subjectstransferred_view' as report_model from (SELECT prn.subject_identifier, prn.site_id, consent_datetime AS consented, v.visit_code, v.report_datetime AS last_visit, prn.report_datetime AS transferred, CAST(CAST((JULIANDAY(prn.report_datetime) - JULIANDAY(consent_datetime)) AS INTEGER) AS REAL) / 30 AS months, eof.offstudy_datetime AS offstudy, eof.report_datetime AS last_seen FROM intecomm_prn_subjecttransfer AS prn LEFT JOIN edc_registration_registeredsubject AS rs ON prn.subject_identifier = rs.subject_identifier LEFT JOIN intecomm_prn_endofstudy AS eof ON prn.subject_identifier = eof.subject_identifier LEFT JOIN (SELECT DISTINCT subject_identifier, LAST_VALUE(report_datetime) OVER w AS report_datetime, LAST_VALUE(visit_code) OVER w AS visit_code FROM visits WINDOW w AS (PARTITION BY \"subject_identifier\")) AS v ON prn.subject_identifier = v.subject_identifier ORDER BY CAST(CAST((JULIANDAY(prn.report_datetime) - JULIANDAY(consent_datetime)) AS INTEGER) AS REAL) / 30 DESC, subject_identifier) as A ORDER BY site_id, subject_identifier",
                "subjects_transferred_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "subjects_transferred_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'intecomm_reports.vl_view' as `report_model` from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(MONTH, baseline_date, vl_date) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "intecomm_reports_vl_view", engine="django.db.backends.mysql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'intecomm_reports.vl_view' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN CAST(crf.report_datetime AS DATE) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "intecomm_reports_vl_view", engine="django.db.backends.postgresql"
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'intecomm_reports.vl_view' as report_model from (SELECT B1.* FROM (SELECT hivreview.*, dx.baseline_date, TIMESTAMPDIFF(vl_date, baseline_date, MONTH) AS m FROM (SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivinitialreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id WHERE NOT vl IS NULL UNION SELECT subject_identifier, crf.site_id, vl AS vl_value, CASE WHEN drawn_date IS NULL THEN DATE(crf.report_datetime) ELSE drawn_date END AS vl_date FROM intecomm_subject_hivreview AS crf LEFT JOIN intecomm_subject_subjectvisit AS v ON v.id = crf.subject_visit_id) AS hivreview LEFT JOIN intecomm_reports_diagnoses AS dx ON dx.subject_identifier = hivreview.subject_identifier WHERE NOT vl_value IS NULL) AS B1) as A ORDER BY site_id, subject_identifier",
                "intecomm_reports_vl_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "", "intecomm_reports_vl_view", engine="django.db.backends.sqlite3"
            ),
            atomic=False,
        ),
    ]



from .assets_form import HealthEconomicsAssetsForm
from .careseeking_a_form import CareseekingAForm
from .careseeking_b_form import CareseekingBForm
from .household_head import HealthEconomicsHouseholdHeadForm
from .income_form import HealthEconomicsIncomeForm
from .patient_form import HealthEconomicsPatientForm
from .property_form import HealthEconomicsPropertyForm



# Generated by Django 4.2.9 on 2024-01-30 01:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0138_alter_bloodresultsfbc_site_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultsins",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="dmreview",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreview",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="glucose_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
    ]



from django.db import models
from django_pandas.managers import DataFrameManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_qareports.model_mixins import QaReportModelMixin
from edc_utils import get_utcnow


class BaseVlSummaryModelMixin(
    UniqueSubjectIdentifierFieldMixin, QaReportModelMixin, models.Model
):
    """A modelmixin for VL data management tables with details of each
    HIV participant's viral load value and status.
    """

    created = models.DateTimeField(default=get_utcnow)

    baseline_date = models.DateField(null=True)

    baseline_vl_date = models.DateField(null=True)

    endline_vl_date = models.DateField(null=True)

    baseline_vl = models.IntegerField(null=True)

    endline_vl = models.IntegerField(null=True)

    offschedule_date = models.DateField(null=True)

    last_vl_date = models.DateField(null=True)

    next_vl_date = models.DateField(null=True)

    expected = models.BooleanField(null=True)

    offset = models.IntegerField(null=True)

    objects = DataFrameManager()

    class Meta:
        abstract = True



from django import forms
from edc_model.widgets import SliderWidget
from intecomm_form_validators.subject import DmMedicationAdherenceFormValidator

from ..models import DmMedicationAdherence
from .mixins import CrfModelFormMixin


class DmMedicationAdherenceForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmMedicationAdherenceFormValidator

    visual_score_slider = forms.CharField(
        label="Visual score for your diabetes medication",
        widget=SliderWidget(attrs={"min": 0, "max": 100}),
    )

    class Meta:
        model = DmMedicationAdherence
        fields = "__all__"
        labels = {
            "last_missed_pill": (
                "When was the last time you missed taking your diabetes medication?"
            ),
            "missed_pill_reason": (
                "If your diabetes medication was missed, select a reason or reasons"
            ),
        }



from .dbviews import EosAdmin, SubjectsTransferredAdmin, VlAdmin
from .diagnoses_admin import DiagnosesAdmin
from .missing_vl_draw_dates_admin import MissingVlDrawDatesAdmin
from .vl_summary6m_admin import VlSummary6mAdmin
from .vl_summary9m_admin import VlSummary9mAdmin



# Generated by Django 5.0.4 on 2024-04-18 01:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0177_alter_careseekingb_accompany_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="careseekinga",
            old_name="travel_time",
            new_name="travel_duration",
        ),
        migrations.RenameField(
            model_name="historicalcareseekinga",
            old_name="travel_time",
            new_name="travel_duration",
        ),
        migrations.AddField(
            model_name="careseekinga",
            name="care_visit_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="careseekinga",
            name="travel_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="careseekinga",
            name="with_hcw_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="careseekingb",
            name="care_visit_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="careseekingb",
            name="travel_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="care_visit_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="travel_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="with_hcw_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicalcareseekingb",
            name="care_visit_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name="historicalcareseekingb",
            name="travel_tdelta",
            field=models.DurationField(blank=True, editable=False, null=True),
        ),
    ]



# Generated by Django 4.2.11 on 2024-07-03 23:52

from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import edc_sites.managers
import edc_utils.date


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_reports", "0007_auto_20240702_0734"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="vl",
            options={
                "managed": False,
                "verbose_name": "Viral loads",
                "verbose_name_plural": "Viral loads",
            },
        ),
        migrations.AlterModelOptions(
            name="vlsummary",
            options={
                "verbose_name": "Viral load summary (endline >= 9m)",
                "verbose_name_plural": "Viral load summary (endline >= 9m)",
            },
        ),
        migrations.AlterModelManagers(
            name="vlsummary",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddField(
            model_name="vlsummary",
            name="expected",
            field=models.BooleanField(null=True),
        ),
        migrations.AddField(
            model_name="vlsummary",
            name="last_vl_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="vlsummary",
            name="next_vl_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="vlsummary",
            name="offschedule_date",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="vlsummary",
            name="offset",
            field=models.IntegerField(null=True),
        ),
        migrations.CreateModel(
            name="VlSummary2",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "report_model",
                    models.CharField(default="intecomm_reports.vlsummary", max_length=50),
                ),
                ("created", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                ("baseline_date", models.DateField(null=True)),
                ("baseline_vl_date", models.DateField(null=True)),
                ("endline_vl_date", models.DateField(null=True)),
                ("baseline_vl", models.IntegerField(null=True)),
                ("endline_vl", models.IntegerField(null=True)),
                ("offschedule_date", models.DateField(null=True)),
                ("last_vl_date", models.DateField(null=True)),
                ("next_vl_date", models.DateField(null=True)),
                ("expected", models.BooleanField(null=True)),
                ("offset", models.IntegerField(null=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Viral load summary (endline >= 6m)",
                "verbose_name_plural": "Viral load summary (endline >= 6m)",
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from __future__ import annotations

from typing import Dict

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.db import IntegrityError
from django.test import TestCase, override_settings, tag
from edc_consent.constants import HOSPITAL_NUMBER
from edc_constants.constants import NO, NOT_APPLICABLE, YES

from intecomm_consent.forms import SubjectConsentTzForm, SubjectConsentUgForm
from intecomm_consent.models import SubjectConsentTz, SubjectConsentUg
from intecomm_lists.models import ScreeningRefusalReasons
from intecomm_screening.models import ConsentRefusal, Site, SubjectScreening
from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin, now
from intecomm_screening.utils import AlreadyRefusedConsentError


class TestSubjectConsent(IntecommTestCaseMixin, TestCase):
    def test_consent_ok(self):
        subject_screening = self.get_subject_screening()
        self.assertEqual(subject_screening.reasons_ineligible, None)
        self.assertTrue(subject_screening.eligible)

        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)

    def test_consenting_when_screened_despite_unwilling_to_screen_raises(self):
        patient_log = self.get_patient_log(willing_to_screen=YES)
        subject_screening = self.get_subject_screening(patient_log=patient_log)
        patient_log.willing_to_screen = NO
        patient_log.screening_refusal_reason = ScreeningRefusalReasons.objects.get(
            name="dont_have_time"
        )
        patient_log.save()
        self.get_subject_consent(subject_screening=subject_screening)
        self.assertRaises(
            IntegrityError, self.get_subject_consent, subject_screening=subject_screening
        )

    def test_consenting_more_than_once_raises(self):
        subject_screening = self.get_subject_screening()
        self.get_subject_consent(subject_screening)
        self.assertRaises(IntegrityError, self.get_subject_consent, subject_screening)

    def test_consenting_after_refusing_consent_raises(self):
        subject_screening = self.get_subject_screening()
        self.get_consent_refusal(screening_identifier=subject_screening.screening_identifier)

        with self.assertRaises(AlreadyRefusedConsentError) as cm:
            self.get_subject_consent(subject_screening)
        self.assertIn(
            "Patient has already refused to consent. "
            f"See {subject_screening.screening_identifier}.",
            str(cm.exception),
        )


class TestSubjectConsentForm(IntecommTestCaseMixin, TestCase):
    def get_consent_data(self, subject_screening: SubjectScreening | None = None) -> Dict:
        subject_screening = subject_screening or self.get_subject_screening()
        return {
            "screening_identifier": subject_screening.screening_identifier,
            "subject_identifier": "123-456-789",
            "report_datetime": now,
            "legal_name": subject_screening.legal_name,
            "familiar_name": subject_screening.familiar_name,
            "initials": subject_screening.initials,
            "gender": subject_screening.gender,
            "language": "en",
            "is_literate": YES,
            "consent_datetime": subject_screening.report_datetime,
            "dob": (now.date() - relativedelta(years=subject_screening.age_in_years)),
            "is_dob_estimated": "-",
            "identity": "123/456/789",
            "identity_type": HOSPITAL_NUMBER,
            "confirm_identity": "123/456/789",
            "is_incarcerated": NO,
            # Review questions
            "consent_reviewed": YES,
            "study_questions": YES,
            "assessment_score": YES,
            "consent_signature": YES,
            "consent_copy": YES,
            # Group
            "group_identifier": "G123-456",
            # Other stuff not included in admin class
            "site": Site.objects.get(id=settings.SITE_ID),
            "citizen": NO,
            "legal_marriage": NOT_APPLICABLE,
            "marriage_certificate": NOT_APPLICABLE,
            "subject_type": "subject",
            "may_store_genetic_samples": NO,
            "may_store_samples": NO,
        }

    @override_settings(SITE_ID=201)
    def test_consent_ok(self):
        subject_screening = self.get_subject_screening()
        consent_form = SubjectConsentTzForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentTz(),
        )
        consent_form.is_valid()
        self.assertEqual(consent_form._errors, {})
        consent_form.save()
        self.assertEqual(SubjectConsentTz.objects.all().count(), 1)

    @override_settings(SITE_ID=101)
    def test_consent_ug_ok(self):
        subject_screening = self.get_subject_screening()
        consent_form = SubjectConsentUgForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentUg(),
        )
        consent_form.is_valid()
        self.assertEqual(consent_form._errors, {})
        consent_form.save()
        self.assertEqual(SubjectConsentUg.objects.all().count(), 1)

    @tag("1")
    @override_settings(SITE_ID=201)
    def test_consent_after_already_consented_raises(self):
        subject_screening = self.get_subject_screening()
        consent_form = SubjectConsentTzForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentTz(),
        )
        consent_form.is_valid()
        self.assertEqual(consent_form._errors, {})
        consent_form.save()
        self.assertEqual(SubjectConsentTz.objects.all().count(), 1)

        consent_form_two = SubjectConsentTzForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentTz(),
        )
        consent_form_two.is_valid()
        self.assertNotEqual(consent_form_two._errors, {})
        self.assertIn("screening_identifier", consent_form_two._errors)
        self.assertEqual(
            ["Subject Consent with this Screening identifier already exists."],
            consent_form_two._errors.get("screening_identifier"),
        )
        with self.assertRaises(ValueError):
            consent_form_two.save()
        self.assertEqual(SubjectConsentTz.objects.all().count(), 1)

    @override_settings(SITE_ID=201)
    def test_consent_after_already_refused_raises(self):
        subject_screening = self.get_subject_screening()
        self.get_consent_refusal(screening_identifier=subject_screening.screening_identifier)
        self.assertEqual(ConsentRefusal.objects.all().count(), 1)

        consent_form = SubjectConsentTzForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentTz(),
        )
        consent_form.is_valid()
        self.assertNotEqual(consent_form._errors, {})
        self.assertIn("__all__", consent_form._errors)
        self.assertIn(
            "Not allowed. Patient has already refused consent. See subject ",
            consent_form._errors.get("__all__")[0],
        )
        self.assertIn(
            str(ConsentRefusal.objects.get(subject_screening_id=subject_screening.id).id),
            consent_form._errors.get("__all__")[0],
        )
        self.assertIn(
            subject_screening.screening_identifier,
            consent_form._errors.get("__all__")[0],
        )
        with self.assertRaises(ValueError):
            consent_form.save()
        self.assertEqual(SubjectConsentTz.objects.all().count(), 0)

    @override_settings(SITE_ID=201)
    def test_consent_if_not_eligible_raises(self):
        subject_screening = self.get_subject_screening()
        subject_screening.in_care_6m = False
        subject_screening.save()
        self.assertFalse(subject_screening.eligible)
        consent_form = SubjectConsentTzForm(
            data=self.get_consent_data(subject_screening=subject_screening),
            initial={"screening_identifier": subject_screening.screening_identifier},
            instance=SubjectConsentTz(),
        )
        consent_form.is_valid()
        self.assertNotEqual(consent_form._errors, {})
        self.assertIn("__all__", consent_form._errors)
        self.assertIn("Subject is not eligible", consent_form._errors.get("__all__")[0])



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING
from zoneinfo import ZoneInfo

from edc_constants.constants import CLINIC, COMMUNITY, DM, HIV, HTN
from edc_dx import Diagnoses
from edc_dx.diagnoses import ClinicalReviewBaselineRequired, InitialReviewRequired
from edc_dx_review.constants import DIET_LIFESTYLE, DRUGS
from edc_he.rule_groups import Predicates as BaseHealthEconomicsPredicates
from edc_visit_schedule.constants import MONTH12
from edc_visit_schedule.utils import is_baseline
from intecomm_rando.constants import COMMUNITY_ARM, FACILITY_ARM
from intecomm_rando.utils import get_assignment_for_subject

from intecomm_subject.models import HtnInitialReview, HtnReview

if TYPE_CHECKING:
    from edc_metadata.model_mixins.creates import CreatesMetadataModelMixin
    from edc_visit_tracking.model_mixins import VisitModelMixin as Base

    class RelatedVisitModel(CreatesMetadataModelMixin, Base):
        pass


class HealthEconomicsPredicates(BaseHealthEconomicsPredicates):
    @staticmethod
    def is_required_by_date(visit, **kwargs) -> bool:
        return visit.report_datetime >= datetime(2023, 7, 14, 1, 00, tzinfo=ZoneInfo("UTC"))

    @staticmethod
    def careseeking_required(visit, **kwargs) -> bool:
        """Returns True if after start date and 12m"""
        is_required_by_date = visit.report_datetime >= datetime(
            2024, 4, 22, 1, 00, tzinfo=ZoneInfo("UTC")
        )
        return is_required_by_date and visit.visit_code == MONTH12


class LocationUpdatePredicates:
    @staticmethod
    def location_needs_update(visit: RelatedVisitModel, **kwargs) -> bool:
        required = False
        try:
            appt_type_name = visit.appointment.appt_type.name
        except AttributeError:
            pass
        else:
            if not is_baseline(visit):
                assignment = (
                    COMMUNITY
                    if get_assignment_for_subject(visit.subject_identifier) == COMMUNITY_ARM
                    else CLINIC
                )
                if assignment != appt_type_name:
                    required = True
        return required


class NextAppointmentPredicates:
    @staticmethod
    def is_required(visit: RelatedVisitModel, **kwargs) -> bool:
        return (
            get_assignment_for_subject(visit.subject_identifier) == FACILITY_ARM
            and visit.visit_code != MONTH12
        )


class MedicationAdherencePredicates:
    @staticmethod
    def diagnoses(visit, **kwargs) -> list[str]:
        dxs = []
        try:
            diagnoses = Diagnoses(
                subject_identifier=visit.subject_identifier,
                report_datetime=visit.report_datetime,
                lte=True,
            )
        except ClinicalReviewBaselineRequired:
            pass
        else:
            try:
                dxs = [name for name in diagnoses.initial_reviews]
            except InitialReviewRequired:
                pass
        return dxs

    @staticmethod
    def is_required_by_date(visit, **kwargs) -> bool:
        return visit.report_datetime >= datetime(2023, 4, 19, 1, 00, tzinfo=ZoneInfo("UTC"))

    @staticmethod
    def is_required_by_date_hiv(visit, **kwargs) -> bool:
        return visit.report_datetime >= datetime(2023, 3, 22, 1, 00, tzinfo=ZoneInfo("UTC"))

    def hiv_adherence_required(self, visit, **kwargs) -> bool:

        return self.is_required_by_date_hiv and HIV.lower() in self.diagnoses(visit, **kwargs)

    def htn_adherence_required(self, visit, **kwargs) -> bool:
        # TODO: not required if managed_by for diet and lifestyle
        if (
            HtnInitialReview.objects.filter(
                subject_visit=visit, managed_by__name=DIET_LIFESTYLE
            )
            .exclude(managed_by__name=DRUGS)
            .exists()
            or HtnReview.objects.filter(subject_visit=visit, managed_by__name=DIET_LIFESTYLE)
            .exclude(managed_by__name=DRUGS)
            .exists()
        ):
            required = False
        else:
            required = self.is_required_by_date and HTN.lower() in self.diagnoses(
                visit, **kwargs
            )
        return required

    def dm_adherence_required(self, visit, **kwargs) -> bool:
        return self.is_required_by_date and DM.lower() in self.diagnoses(visit, **kwargs)



# Generated by Django 4.1.2 on 2022-12-02 22:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0002_delete_reasonsfortesting_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="DmManagement",
            fields=[
                (
                    "name",
                    models.CharField(
                        db_index=True,
                        help_text="This is the stored value, required",
                        max_length=250,
                        unique=True,
                        verbose_name="Stored value",
                    ),
                ),
                (
                    "plural_name",
                    models.CharField(max_length=250, null=True, verbose_name="Plural name"),
                ),
                (
                    "display_name",
                    models.CharField(
                        db_index=True,
                        help_text="(suggest 40 characters max.)",
                        max_length=250,
                        unique=True,
                        verbose_name="Name",
                    ),
                ),
                (
                    "display_index",
                    models.IntegerField(
                        db_index=True,
                        default=0,
                        help_text="Index to control display order if not alphabetical, not required",
                        verbose_name="display index",
                    ),
                ),
                (
                    "field_name",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Not required",
                        max_length=25,
                        null=True,
                    ),
                ),
                ("version", models.CharField(default="1.0", editable=False, max_length=35)),
                ("id", models.AutoField(primary_key=True, serialize=False)),
            ],
            options={
                "verbose_name": "Diabetes Management",
                "verbose_name_plural": "Diabetes Management",
                "ordering": ["display_index", "display_name"],
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
            },
        ),
        migrations.AddIndex(
            model_name="dmmanagement",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="intecomm_li_id_b4d41f_idx",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-26 03:40

from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0012_alter_historicalpatientcall_last_attend_clinic_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healthfacility",
            name="distance",
            field=models.IntegerField(
                help_text="In km. If within the integrated-care clinic facility type '0'",
                verbose_name="Approximate distance from integrated-care clinic",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealthfacility",
            name="distance",
            field=models.IntegerField(
                help_text="In km. If within the integrated-care clinic facility type '0'",
                verbose_name="Approximate distance from integrated-care clinic",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="hospital_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                db_index=True,
                help_text="Must be unique (Encryption: RSA local)",
                max_length=71,
                verbose_name="Hospital identifier",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="hospital_identifier",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Must be unique (Encryption: RSA local)",
                max_length=71,
                unique=True,
                verbose_name="Hospital identifier",
            ),
        ),
    ]



from django.contrib import admin

from ..admin_site import intecomm_reports_admin
from ..models import VlSummary9m
from .modeladmin_mixins import VlSummaryModelAdminMixin


@admin.register(VlSummary9m, site=intecomm_reports_admin)
class VlSummary9mAdmin(VlSummaryModelAdminMixin, admin.ModelAdmin):

    report_model = "intecomm_reports.vlsummary9m"
    endline_months = 9



from django.contrib import admin
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ..admin_site import intecomm_reports_admin
from ..models import Diagnoses


@admin.register(Diagnoses, site=intecomm_reports_admin)
class DiagnosesAdmin(
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    qa_report_list_display_insert_pos = 2
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "subject_identifier",
        "baseline_date",
        "hiv",
        "htn",
        "dm",
        "hiv_dx_date",
        "htn_dx_date",
        "dm_dx_date",
        "hiv_dx_days",
        "htn_dx_days",
        "dm_dx_days",
    ]

    list_filter = [
        ScheduleStatusListFilter,
        "baseline_date",
        "hiv",
        "htn",
        "dm",
    ]

    readonly_fields = (
        "subject_identifier",
        "baseline_date",
        "hiv",
        "htn",
        "dm",
        "hiv_dx_date",
        "htn_dx_date",
        "dm_dx_date",
        "hiv_dx_days",
        "htn_dx_days",
        "dm_dx_days",
        "site",
    )



# Generated by Django 4.2.1 on 2023-06-30 02:49

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_consent", "0007_alter_historicalsubjectconsent_language_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="screening_identifier",
            field=models.CharField(
                db_index=True,
                max_length=50,
                validators=[django.core.validators.RegexValidator("^[A-Z0-9]+$")],
                verbose_name="Screening identifier",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="screening_identifier",
            field=models.CharField(
                max_length=50,
                unique=True,
                validators=[django.core.validators.RegexValidator("^[A-Z0-9]+$")],
                verbose_name="Screening identifier",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-02 22:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0026_alter_historicalpatientlog_filing_identifier_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalpatientlog",
            name="printed",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="patientlog",
            name="printed",
            field=models.BooleanField(default=False),
        ),
    ]



from __future__ import annotations

from django.contrib import admin
from django.template.loader import render_to_string
from edc_locator.admin_site import edc_locator_admin
from edc_locator.modeladmin_mixins import SubjectLocatorModelAdminMixin
from edc_locator.models import SubjectLocator as DefaultSubjectLocator
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from intecomm_prn.admin_site import intecomm_prn_admin
from intecomm_prn.forms import SubjectLocatorForm
from intecomm_prn.models import SubjectLocator
from intecomm_screening.models import PatientLog

edc_locator_admin.unregister(DefaultSubjectLocator)


@admin.register(SubjectLocator, site=intecomm_prn_admin)
class SubjectLocatorAdmin(
    SubjectLocatorModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    list_per_page = 5

    form = SubjectLocatorForm

    @admin.display(description="Contacts")
    def contacts(self, obj):
        patient_log = PatientLog.objects.get(subject_identifier=obj.subject_identifier)
        context = dict(
            patient_log=patient_log,
            subject_cell=obj.subject_cell,
            subject_cell_alt=obj.subject_cell_alt,
            subject_phone=obj.subject_phone,
            subject_phone_alt=obj.subject_phone_alt,
        )
        return render_to_string(
            "intecomm_prn/changelist_locator_contacts.html", context=context
        )



# Generated by Django 4.2.11 on 2024-05-02 00:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0182_dmcurrentconditions"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="dm",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="dm_dx_days",
            field=models.IntegerField(
                default=None, help_text="Number of days relative to baseline", null=True
            ),
        ),
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="hiv",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="hiv_dx_days",
            field=models.IntegerField(
                default=None, help_text="Number of days relative to baseline", null=True
            ),
        ),
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="htn",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="dmcurrentconditions",
            name="htn_dx_days",
            field=models.IntegerField(
                default=None, help_text="Number of days relative to baseline", null=True
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-12-06 23:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0005_alter_historicalsocialharms_coworkers_impact_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="healtheconomics",
            options={
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Health Economics",
                "verbose_name_plural": "Health Economics",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhealtheconomics",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Health Economics",
                "verbose_name_plural": "historical Health Economics",
            },
        ),
    ]



# Generated by Django 4.2.11 on 2024-04-22 16:49
from django.db import migrations
from django.db.migrations import RunPython
from tqdm import tqdm

from intecomm_lists.models import HtnManagement


def update_m2m_from_charfield(apps, schema_editor):
    model_cls = apps.get_model("intecomm_subject", "htnreview")
    qs = model_cls.objects.all()
    total = qs.count()
    for obj in tqdm(qs, total=total):
        htn_management = HtnManagement.objects.get(name=obj.managed_by_old)
        obj.managed_by.add(htn_management.id)
        obj.save()


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0180_historicalhtnreview_managed_by_other_and_more"),
    ]

    operations = [RunPython(update_m2m_from_charfield)]



# Generated by Django 4.2.1 on 2023-07-07 19:32

from django.db import migrations
import edc_action_item.managers
import edc_adverse_event.model_mixins.death_report.death_report_tmg_model_mixin


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_ae",
            "0004_alter_aefollowup_options_alter_aeinitial_options_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="aefollowup",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_action_item.managers.ActionIdentifierSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="aeinitial",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_action_item.managers.ActionIdentifierSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="aesusar",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_action_item.managers.ActionIdentifierSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="aetmg",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_action_item.managers.ActionIdentifierSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="deathreport",
            managers=[
                ("objects", edc_action_item.managers.ActionIdentifierModelManager()),
                ("on_site", edc_action_item.managers.ActionIdentifierSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="deathreporttmg",
            managers=[
                (
                    "objects",
                    edc_adverse_event.model_mixins.death_report.death_report_tmg_model_mixin.DeathReportTmgManager(),
                ),
                (
                    "on_site",
                    edc_adverse_event.model_mixins.death_report.death_report_tmg_model_mixin.DeathReportTmgSiteManager(),
                ),
            ],
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_dx import get_diagnosis_labels_prefixes
from edc_dx_review.fieldsets import get_clinical_review_cond_fieldsets
from edc_dx_review.radio_fields import get_clinical_review_cond_radio_fields

from ..admin_site import intecomm_subject_admin
from ..forms import ClinicalReviewForm
from ..models import ClinicalReview
from .fieldsets import treatment_pay_methods_fieldset_tuple
from .modeladmin_mixins import CrfModelAdmin


@admin.register(ClinicalReview, site=intecomm_subject_admin)
class ClinicalReviewAdmin(CrfModelAdmin):
    form = ClinicalReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        *get_clinical_review_cond_fieldsets(),
        ("Complications", {"fields": ("complications",)}),
        treatment_pay_methods_fieldset_tuple,
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "complications": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "health_insurance": admin.VERTICAL,
        "patient_club": admin.VERTICAL,
        **get_clinical_review_cond_radio_fields(),
    }

    filter_horizontal = [f"{cond}_reason" for cond in get_diagnosis_labels_prefixes()]



# Generated by Django 4.2.3 on 2023-08-03 03:34
from django.db import migrations
from edc_list_data import PreloadData
from tqdm import tqdm

from intecomm_lists.list_data import list_data


def update_religion_on_he(apps, schema_editor):
    list_model_cls = apps.get_model("edc_he.religions")
    list_model_cls.objects.filter(name="seventh-day-adventist").update(
        name="seventh_day_adventist"
    )
    list_model_cls.objects.filter(name="assemblies-of-god").update(name="tz_assemblies_of_god")
    list_model_cls.objects.filter(name="african-inland-church").update(
        name="african_inland_church"
    )
    list_model_cls.objects.filter(name="none").update(name="no_religion")
    ld = {"edc_he.religions": list_data.get("edc_he.religions")}
    PreloadData(list_data=ld, apps=apps)

    householdhead_cls = apps.get_model("intecomm_subject.healtheconomicshouseholdhead")
    total = householdhead_cls.objects.all().count()
    for obj in tqdm(householdhead_cls.objects.all(), total=total):
        list_model_obj = list_model_cls.objects.get(name=obj.hoh_religion_old)
        obj.hoh_religion_id = list_model_obj.id
        obj.save_base(update_fields="hoh_religion")

    patient_cls = apps.get_model("intecomm_subject.healtheconomicspatient")
    total = patient_cls.objects.all().count()
    for obj in tqdm(patient_cls.objects.all(), total=total):
        list_model_obj = list_model_cls.objects.get(name=obj.pat_religion_old)
        obj.pat_religion_id = list_model_obj.id
        obj.save_base(update_fields="pat_religion")


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0078_alter_healtheconomicshouseholdhead_hoh_religion_old_and_more",
        ),
    ]

    operations = [migrations.RunPython(update_religion_on_he)]



from django import forms
from edc_form_validators import FormValidatorMixin
from intecomm_form_validators.screening import PatientGroupRandoFormValidator

from ..models import PatientGroupRando


class PatientGroupRandoForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = PatientGroupRandoFormValidator

    # TODO: check if any in more than one group

    name = forms.CharField(
        label="Group name",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = PatientGroupRando
        fields = [
            "name",
            "randomized_datetime",
            "randomize_now",
            "confirm_randomize_now",
            "group_identifier",
        ]



# Generated by Django 5.0.4 on 2024-04-12 00:53

import django.core.validators
import edc_model_fields.fields.custom_django_fields
import edc_model_fields.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_lists", "0012_visitreasons_custom_name"),
        ("intecomm_subject", "0164_alter_careseekinga_money_source_main_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="careseekinga",
            name="care_visit_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDCONDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="care_visit_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDCONDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultsins",
            name="fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="care_visit_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on healthworker and consultation fees, enter `0`",
                metadata="FFEECOST1",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much money did you spend on healthworker and consultation fees during today’s visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FFACTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend during your visit today -- from arrival to this place until the end of your visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="food_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on food, drink or other refreshments, enter `0`",
                metadata="FFOODCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Thinking about yourself and anyone that accompanied you, how much did you have to pay for food, drink or other refreshments during your travel or during your visit? (e.g. food, drink, etc.)",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_conditions",
            field=edc_model_fields.fields.custom_django_fields.ManyToManyField2(
                blank=True,
                metadata="FMEDCOND1",
                to="intecomm_lists.conditions",
                verbose_name="What were the medicines were for?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_prescribed",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                metadata="FMED1",
                verbose_name="Were you prescribed any medicines during today’s visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="travel_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on travel, enter `0`",
                metadata="FTRAVCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Thinking about yourself and anyone that accompanied you, how much was spent on travel from your home to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="travel_time",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="wait_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FWAITIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend waiting?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="with_hcw_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FWORKTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend with the healthcare worker?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="in hours and minutes (format HH:MM)",
                max_length=5,
                metadata="FOUTTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="Roughly how much time did you spend during your last/most recent visit?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="travel_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="in hours and minutes (format HH:MM)",
                max_length=5,
                metadata="FOUTTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to get there?",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did they fast (in hours and minutes)?",
            ),
        ),
        migrations.AlterField(
            model_name="dmreview",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultsins",
            name="fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="care_visit_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on healthworker and consultation fees, enter `0`",
                metadata="FFEECOST1",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much money did you spend on healthworker and consultation fees during today’s visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FFACTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend during your visit today -- from arrival to this place until the end of your visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="food_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on food, drink or other refreshments, enter `0`",
                metadata="FFOODCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Thinking about yourself and anyone that accompanied you, how much did you have to pay for food, drink or other refreshments during your travel or during your visit? (e.g. food, drink, etc.)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_prescribed",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                metadata="FMED1",
                verbose_name="Were you prescribed any medicines during today’s visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="travel_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                help_text="In local currency. If nothing spent on travel, enter `0`",
                metadata="FTRAVCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Thinking about yourself and anyone that accompanied you, how much was spent on travel from your home to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="travel_time",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to reach here?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="wait_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FWAITIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend waiting?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="with_hcw_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="something like 1h20m, 11h5m, etc",
                max_length=5,
                metadata="FWORKTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How much time did you spend with the healthcare worker?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="care_visit_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="in hours and minutes (format HH:MM)",
                max_length=5,
                metadata="FOUTTIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="Roughly how much time did you spend during your last/most recent visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="travel_duration",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                help_text="in hours and minutes (format HH:MM)",
                max_length=5,
                metadata="FOUTTRATIME1",
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did it take you to get there?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long did they fast (in hours and minutes)?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreview",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="glucose_fasting_duration_str",
            field=models.CharField(
                blank=True,
                help_text="As reported by patient. Duration of fast. Format is `HHhMMm`. For example 1h23m, 12h7m, etc",
                max_length=8,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "^([0-9]{1,3}h([0-5]?[0-9]m)?)$",
                        message="Invalid format. Please insert two numeric values followed by “h” for hours, and two numeric values followed by “m” for minute. For example, 01h20m, 00h35m, and so on",
                    )
                ],
                verbose_name="How long have they fasted in hours and/or minutes?",
            ),
        ),
    ]



from django.db import models
from edc_constants.constants import NOT_APPLICABLE
from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_dx_review.model_mixins import (
    dx_initial_review_model_mixin_factory,
    rx_initial_review_model_mixin_factory,
)
from edc_model.models import BaseUuidModel
from edc_model_fields.fields import OtherCharField

from intecomm_lists.models import HtnManagement

from ..choices import HTN_MANAGEMENT
from ..model_mixins import CrfModelMixin


class HtnInitialReview(
    dx_initial_review_model_mixin_factory("dx"),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="medicines for hypertension"
    ),
    SingletonCrfModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    diagnosis_label = "hypertension"
    managed_by = models.ManyToManyField(
        HtnManagement,
        verbose_name="How is the patient's hypertension managed?",
    )

    managed_by_other = OtherCharField()

    managed_by_old = models.CharField(
        verbose_name="How is the patient's hypertension managed?",
        max_length=15,
        choices=HTN_MANAGEMENT,
        default=NOT_APPLICABLE,
        blank=True,
        help_text="If patient is taking medication, complete the Drug Refill CRF.",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Hypertension Initial Review"
        verbose_name_plural = "Hypertension Initial Reviews"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



# Generated by Django 4.1.7 on 2023-04-19 14:01

from django.db import migrations, models
import edc_model.validators.date


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0031_alter_clinicalreviewbaseline_dm_dx_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalnextappointment",
            name="best_visit_code",
            field=models.CharField(
                choices=[
                    ("100", "100"),
                    ("1010", "1010"),
                    ("1020", "1020"),
                    ("1030", "1030"),
                    ("1040", "1040"),
                    ("1050", "1050"),
                    ("1060", "1060"),
                    ("1070", "1070"),
                    ("1080", "1080"),
                    ("1090", "1090"),
                    ("1100", "1100"),
                    ("1110", "1110"),
                    ("1120", "1120"),
                ],
                help_text=(
                    "Click save to let the EDC suggest. Once selected, previous visits will be flagged as not required.",
                ),
                max_length=15,
                null=True,
                verbose_name="Which study visit code is closest to this appointment date",
            ),
        ),
        migrations.AddField(
            model_name="nextappointment",
            name="best_visit_code",
            field=models.CharField(
                choices=[
                    ("100", "100"),
                    ("1010", "1010"),
                    ("1020", "1020"),
                    ("1030", "1030"),
                    ("1040", "1040"),
                    ("1050", "1050"),
                    ("1060", "1060"),
                    ("1070", "1070"),
                    ("1080", "1080"),
                    ("1090", "1090"),
                    ("1100", "1100"),
                    ("1110", "1110"),
                    ("1120", "1120"),
                ],
                help_text=(
                    "Click save to let the EDC suggest. Once selected, previous visits will be flagged as not required.",
                ),
                max_length=15,
                null=True,
                verbose_name="Which study visit code is closest to this appointment date",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="rx_init",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Has the patient started medicines for diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="rx_init_calculated_date",
            field=models.DateField(
                editable=False,
                help_text="Calculated based on response to `rx_init_ago`",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Estimated date started medicines for diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started medicines for diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="rx_init_date_is_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="N/A",
                editable=False,
                max_length=15,
                verbose_name="Was medicines for diabetes start date estimated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="rx_init",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Has the patient started medicines for diabetes?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="rx_init_calculated_date",
            field=models.DateField(
                editable=False,
                help_text="Calculated based on response to `rx_init_ago`",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Estimated date started medicines for diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started medicines for diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="rx_init_date_is_estimated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="N/A",
                editable=False,
                max_length=15,
                verbose_name="Was medicines for diabetes start date estimated?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivinitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivinitialreview",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started medicines for hypertension",
            ),
        ),
        migrations.AlterField(
            model_name="historicalviralloadresult",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="=",
                max_length=10,
                verbose_name="Quantifier",
            ),
        ),
        migrations.AlterField(
            model_name="hivinitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started antiretroviral therapy (ART)",
            ),
        ),
        migrations.AlterField(
            model_name="hivinitialreview",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="rx_init_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date started medicines for hypertension",
            ),
        ),
        migrations.AlterField(
            model_name="viralloadresult",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="=",
                max_length=10,
                verbose_name="Quantifier",
            ),
        ),
    ]



from __future__ import annotations

from django.contrib.admin import SimpleListFilter
from django.db.models import Q, QuerySet
from django.utils.translation import gettext_lazy as _
from edc_constants.constants import DM, HIV, HTN, NCD, NO, TBD, YES
from edc_model_admin.list_filters import FutureDateListFilter, PastDateListFilter
from edc_protocol.research_protocol_config import ResearchProtocolConfig

HIV_ONLY = "HIV_ONLY"
NCD_ONLY = "NCD_ONLY"
HTN_ONLY = "HTN_ONLY"
DM_ONLY = "DM_ONLY"


class NextApptListFilter(FutureDateListFilter):
    title = _("Next Appt")

    parameter_name = "next_appt_date"
    field_name = "next_appt_date"


class LastApptListFilter(PastDateListFilter):
    title = _("Last Appt")

    parameter_name = "last_appt_date"
    field_name = "last_appt_date"


class InPatientGroup(SimpleListFilter):
    title = "In a group"

    parameter_name = "patient_group"
    field_name = "patient_group"

    def lookups(self, request, model_admin) -> tuple:
        return (
            (YES, "Yes"),
            (NO, "No"),
        )

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = None
        if self.value() == YES:
            qs = queryset.filter(patientgroup__isnull=False)
        if self.value() == NO:
            qs = queryset.filter(patientgroup__isnull=True)
        return qs


class StableListFilter(SimpleListFilter):
    title = "Stable"

    parameter_name = "stable"
    field_name = "stable"

    def lookups(self, request, model_admin) -> tuple:
        return (
            (YES, "Yes"),
            (NO, "No"),
            (TBD, "To be determined (TBD)"),
        )

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = None
        if self.value() == YES:
            qs = queryset.filter(stable=YES)
        if self.value() == NO:
            qs = queryset.filter(stable=NO)
        if self.value() == TBD:
            qs = queryset.filter(stable=TBD)
        return qs


class ScreenedListFilter(SimpleListFilter):
    title = "Screened"

    parameter_name = "screening_identifier"
    field_name = "screening_identifier"

    def lookups(self, request, model_admin) -> tuple:
        return (
            (YES, "Yes"),
            (NO, "No"),
        )

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = None
        if self.value() == YES:
            qs = queryset.filter(subjectscreening__screening_identifier__isnull=False)
        if self.value() == NO:
            qs = queryset.filter(subjectscreening__screening_identifier__isnull=True)
        return qs


class ConsentedListFilter(SimpleListFilter):
    title = "Consented"

    parameter_name = "subject_identifier"
    field_name = "subject_identifier"

    def lookups(self, request, model_admin) -> tuple:
        return (
            (YES, "Yes"),
            (NO, "No"),
        )

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = None
        number = ResearchProtocolConfig().protocol_number
        if self.value() == YES:
            qs = queryset.filter(
                subjectscreening__subject_identifier__isnull=False,
                subjectscreening__subject_identifier__startswith=number,
            )
        if self.value() == NO:
            qs = queryset.filter(
                Q(subjectscreening__subject_identifier__isnull=False)
                & ~Q(subjectscreening__subject_identifier__startswith=number),
            )
        return qs


class DxListFilter(SimpleListFilter):
    title = "DX"

    parameter_name = "conditions"
    field_name = "conditions"

    def lookups(self, request, model_admin) -> tuple:
        return (
            (NCD, "NCD"),
            (HIV, "HIV"),
            (NCD_ONLY, "NCD only"),
            (DM_ONLY, "DM only"),
            (HIV_ONLY, "HIV only"),
            (HTN_ONLY, "HTN only"),
        )

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = None
        if self.value() == NCD:
            qs = queryset.filter(conditions__name__in=[DM, HTN])
        if self.value() == HIV:
            qs = queryset.filter(conditions__name__in=[HIV])
        if self.value() == NCD_ONLY:
            qs = queryset.filter(conditions__name__in=[DM, HTN]).exclude(
                conditions__name__in=[HIV]
            )
        if self.value() == HIV_ONLY:
            qs = queryset.filter(conditions__name__in=[HIV]).exclude(
                conditions__name__in=[HTN, DM]
            )
        if self.value() == HTN_ONLY:
            qs = queryset.filter(conditions__name__in=[HTN]).exclude(
                conditions__name__in=[HIV, DM]
            )
        if self.value() == DM_ONLY:
            qs = queryset.filter(conditions__name__in=[DM]).exclude(
                conditions__name__in=[HIV, HTN]
            )
        return qs


class AttendDateListFilter(FutureDateListFilter):
    title = "Next Appt"

    parameter_name = "next_appt_date"
    field_name = "patientcall__next_appt_date"

    def queryset(self, request, queryset) -> QuerySet | None:
        qs = super().queryset(request, queryset)
        if qs:
            # dedup
            pk_set = set([obj.pk for obj in qs])
            return qs.model.objects.filter(pk__in=pk_set)
        return qs



from django.contrib import admin
from django.db.models import QuerySet
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_reports_admin
from ..models import MissingVlDrawDates
from ..vl import get_missing_drawn_dates_df
from .list_filters import ScheduleStatusListFilter


@admin.register(MissingVlDrawDates, site=intecomm_reports_admin)
class MissingVlDrawDatesAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ["site", "subject_identifier"]

    list_display = [
        "dashboard",
        "subject_identifier",
        "baseline_date",
        "visit_code",
        "visit_date",
        "vl",
        "site",
        "created",
    ]

    list_filter = [
        ScheduleStatusListFilter,
        "visit_code",
    ]

    search_fields = ["id", "subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        get_missing_drawn_dates_df(model="intecomm_reports.missingvldrawdates")
        return super().get_queryset(request)



# Generated by Django 4.2.10 on 2024-02-22 07:18

from django.db import migrations
import edc_model_fields.fields.custom_django_fields


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_subject",
            "0146_rename_seek_inpatient_visits_careseekingb_inpatient_days_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="careseekinga",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                metadata="FACMP1",
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                metadata="FOUTACMPNUM1",
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                metadata="FACMP1",
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="accompany_num",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                metadata="FOUTACMPNUM1",
                verbose_name="Number of people who accompanied you here today",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-22 19:59

import django.core.validators
from django.db import migrations, models
import django_crypto_fields.fields.encrypted_char_field
import edc_model.models.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0007_historicalpatientlog_last_4_contact_number_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalpatientlog",
            old_name="last_routine_appt_date",
            new_name="last_appt_date",
        ),
        migrations.RenameField(
            model_name="historicalpatientlog",
            old_name="next_routine_appt_date",
            new_name="next_appt_date",
        ),
        migrations.RenameField(
            model_name="patientlog",
            old_name="last_routine_appt_date",
            new_name="last_appt_date",
        ),
        migrations.RenameField(
            model_name="patientlog",
            old_name="next_routine_appt_date",
            new_name="next_appt_date",
        ),
        migrations.RemoveField(
            model_name="historicalpatientcall",
            name="attend_date",
        ),
        migrations.RemoveField(
            model_name="patientcall",
            name="attend_date",
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="last_appt_date",
            field=models.DateField(
                blank=True,
                help_text="This may be helpful when updating health records",
                null=True,
                verbose_name="When did the patient last seek care",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="last_attend_clinic",
            field=models.CharField(
                blank=True,
                choices=[],
                help_text="This may be helpful when updating health records",
                max_length=25,
                null=True,
                verbose_name="Where did the patient last seek care",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="last_attend_clinic_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatientcall",
            name="next_appt_date",
            field=models.DateField(
                blank=True, null=True, verbose_name="When will the patient next attend"
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="last_appt_date",
            field=models.DateField(
                blank=True,
                help_text="This may be helpful when updating health records",
                null=True,
                verbose_name="When did the patient last seek care",
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="last_attend_clinic",
            field=models.CharField(
                blank=True,
                choices=[],
                help_text="This may be helpful when updating health records",
                max_length=25,
                null=True,
                verbose_name="Where did the patient last seek care",
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="last_attend_clinic_other",
            field=edc_model.models.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="patientcall",
            name="next_appt_date",
            field=models.DateField(
                blank=True, null=True, verbose_name="When will the patient next attend"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                db_index=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="familiar_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="How should we refer to you? (if we speak to you)",
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="legal_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Ensure full name consists of letters only in upper case separated by single spaces",
                        regex="^(([A-Z]+ )*[A-Z]+)?$",
                    )
                ],
                verbose_name="Full name",
            ),
        ),
    ]



from django.contrib import admin

from ..admin_site import intecomm_reports_admin
from ..models import VlSummary6m
from .modeladmin_mixins import VlSummaryModelAdminMixin


@admin.register(VlSummary6m, site=intecomm_reports_admin)
class VlSummary6mAdmin(VlSummaryModelAdminMixin, admin.ModelAdmin):

    report_model = "intecomm_reports.vlsummary6m"
    endline_months = 6



from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin


class HomeView(EdcViewMixin, NavbarViewMixin, TemplateView):
    template_name = f"intecomm_edc/bootstrap{settings.EDC_BOOTSTRAP}/home.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "home"



# Generated by Django 4.1.7 on 2023-04-04 00:41

import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0019_remove_historicalhtninitialreview_med_start_date_estimated_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalhtninitialreview",
            old_name="managed_by",
            new_name="managed_by_old",
        ),
        migrations.RenameField(
            model_name="htninitialreview",
            old_name="managed_by",
            new_name="managed_by_old",
        ),
        migrations.RemoveField(
            model_name="historicalhtninitialreview",
            name="med_start_date",
        ),
        migrations.RemoveField(
            model_name="htninitialreview",
            name="med_start_date",
        ),
        migrations.AlterField(
            model_name="dminitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldminitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhivinitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="med_start_estimated_date",
            field=models.DateField(
                editable=False,
                help_text="auto completed",
                null=True,
                verbose_name="Estimated medication start date",
            ),
        ),
        migrations.AlterField(
            model_name="hivinitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="dx_date",
            field=models.DateField(
                blank=True,
                help_text="If possible, provide the exact date here instead of estimating.",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient diagnosed",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="med_start_estimated_date",
            field=models.DateField(
                editable=False,
                help_text="auto completed",
                null=True,
                verbose_name="Estimated medication start date",
            ),
        ),
    ]



from edc_crf.model_mixins import CrfModelMixin as BaseCrfModelMixin
from edc_crf.model_mixins import CrfStatusModelMixin


class CrfModelMixin(CrfStatusModelMixin, BaseCrfModelMixin):
    class Meta(CrfStatusModelMixin.Meta, BaseCrfModelMixin.Meta):
        abstract = True
        indexes = BaseCrfModelMixin.Meta.indexes



# Generated by Django 4.2.11 on 2024-05-31 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_subject", "0184_rename_dmcurrentconditions_currentconditions_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="CurrentConditions",
            new_name="Diagnoses",
        ),
        migrations.RenameIndex(
            model_name="diagnoses",
            new_name="intecomm_su_modifie_b03a1f_idx",
            old_name="intecomm_su_modifie_490726_idx",
        ),
        migrations.RenameIndex(
            model_name="diagnoses",
            new_name="intecomm_su_user_mo_24138d_idx",
            old_name="intecomm_su_user_mo_15c1bf_idx",
        ),
    ]



import datetime as dt
import io
import os
import sys
from zoneinfo import ZoneInfo

import time_machine
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.test import TestCase, override_settings
from edc_constants.constants import DM, HIV, HTN
from edc_randomization.site_randomizers import site_randomizers
from edc_utils import get_utcnow
from intecomm_form_validators import IN_FOLLOWUP
from intecomm_rando.models import RandomizationList
from intecomm_rando.randomizers import Randomizer
from intecomm_rando.utils import get_assignment_for_subject

from intecomm_group.models import PatientGroup
from intecomm_group.utils import add_subjects_to_group
from intecomm_screening.models import PatientLog
from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin

utc_tz = ZoneInfo("UTC")


@override_settings(
    EDC_RANDOMIZATION_LIST_PATH=os.path.join(
        settings.BASE_DIR, "intecomm_subject", "tests", "etc", "community_only"
    )
)
@time_machine.travel(dt.datetime(2019, 5, 11, 8, 00, tzinfo=utc_tz))
class TestAddToCommunityArmGroup(IntecommTestCaseMixin, TestCase):
    patient_group = None
    import_randomization_list = False
    sid_count_for_tests = 2
    report_datetime: dt.datetime = None
    patient_group_name = "BRANDX"

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.report_datetime = get_utcnow() - relativedelta(days=10)
        site_randomizers._registry = {}
        Randomizer.randomizationlist_folder = settings.EDC_RANDOMIZATION_LIST_PATH
        site_randomizers.register(Randomizer)
        randomizer_cls = site_randomizers.get("default")
        randomizer_cls.import_list(verbose=False, sid_count_for_tests=cls.sid_count_for_tests)
        cls.patient_group = cls.get_randomized_patient_group(
            report_datetime=cls.report_datetime, group_name=cls.patient_group_name
        )

    def setUp(self):
        self.assertTrue(self.patient_group.randomized)
        patient_log = self.patient_group.patients.filter(conditions__name__in=[HTN]).first()
        self.subject_identifier = patient_log.subject_identifier

    def test_add_subjects_to_group(self):
        patient_group = PatientGroup.objects.get(name=self.patient_group_name)
        assignment = RandomizationList.objects.get(
            group_identifier=patient_group.group_identifier
        ).assignment

        self.assertEqual(patient_group.status, IN_FOLLOWUP)
        patient_count = patient_group.patients.all().count()
        hiv_count = (
            patient_group.patients.filter(conditions__name__in=[HIV])
            .exclude(conditions__name__in=[HTN, DM])
            .count()
        )
        htn_count = (
            patient_group.patients.filter(conditions__name__in=[HTN])
            .exclude(conditions__name__in=[HIV, DM])
            .count()
        )
        dm_count = (
            patient_group.patients.filter(conditions__name__in=[DM])
            .exclude(conditions__name__in=[HIV, HTN])
            .count()
        )

        pat1 = self.get_consented_patient(
            condition_name=HIV, report_datetime=self.report_datetime
        )
        pat2 = self.get_consented_patient(
            condition_name=HTN, report_datetime=self.report_datetime
        )
        pat3 = self.get_consented_patient(
            condition_name=DM, report_datetime=self.report_datetime
        )
        subject_identifiers = [
            pat1.subject_identifier,
            pat2.subject_identifier,
            pat3.subject_identifier,
        ]
        add_subjects_to_group(self.patient_group_name, subject_identifiers)

        patient_group = PatientGroup.objects.get(name=self.patient_group_name)
        self.assertEqual(patient_group.patients.all().count(), patient_count + 3)

        new_hiv_count = (
            patient_group.patients.filter(conditions__name__in=[HIV])
            .exclude(conditions__name__in=[HTN, DM])
            .count()
        )
        new_htn_count = (
            patient_group.patients.filter(conditions__name__in=[HTN])
            .exclude(conditions__name__in=[HIV, DM])
            .count()
        )
        new_dm_count = (
            patient_group.patients.filter(conditions__name__in=[DM])
            .exclude(conditions__name__in=[HIV, HTN])
            .count()
        )

        self.assertEqual(patient_group.patients.all().count(), patient_count + 3)
        self.assertEqual(dm_count + 1, new_dm_count)
        self.assertEqual(htn_count + 1, new_htn_count)
        self.assertEqual(hiv_count + 1, new_hiv_count)
        self.assertEqual(patient_group.status, IN_FOLLOWUP)

        for subject_identifier in subject_identifiers:
            obj = PatientLog.objects.get(subject_identifier=subject_identifier)
            self.assertEqual(obj.group_identifier, patient_group.group_identifier)
            self.assertEqual(get_assignment_for_subject(subject_identifier), assignment)

        # try to add again
        add_subjects_to_group(self.patient_group_name, subject_identifiers)

        # no change
        self.assertEqual(patient_group.patients.all().count(), patient_count + 3)
        self.assertEqual(dm_count + 1, new_dm_count)
        self.assertEqual(htn_count + 1, new_htn_count)
        self.assertEqual(hiv_count + 1, new_hiv_count)
        self.assertEqual(patient_group.status, IN_FOLLOWUP)

        # try unknown group
        captured_output = io.StringIO()
        sys.stdout = captured_output
        try:
            add_subjects_to_group("BLAH", subject_identifiers)
        finally:
            sys.stdout = sys.__stdout__
            self.assertIn("PatientGroup does not exist", captured_output.getvalue())

        # try unknown subject
        captured_output = io.StringIO()
        sys.stdout = captured_output
        try:
            add_subjects_to_group(self.patient_group_name, ["1234"])
        finally:
            sys.stdout = sys.__stdout__
            self.assertIn("skipping", captured_output.getvalue())



from django.db import models
from django_db_views.db_view import DBView
from edc_qareports.model_mixins import QaReportModelMixin

from .view_definition import get_view_definition


class Eos(QaReportModelMixin, DBView):

    visit_datetime = models.DateTimeField(null=True, blank=True)
    timepoint = models.DecimalField(null=True, blank=True, decimal_places=2, max_digits=10)
    visit_code = models.CharField(max_length=25, null=True)
    visit_code_sequence = models.IntegerField(null=True)
    offstudy_reason = models.CharField(max_length=200, null=True, blank=True)
    offstudy_datetime = models.DateTimeField(null=True, blank=True)
    m = models.IntegerField(verbose_name="months", null=True)
    schedule_status = models.CharField(max_length=25, null=True, blank=True)

    view_definition = get_view_definition()

    class Meta:
        managed = False
        db_table = "eos_view"
        verbose_name = "End of study"
        verbose_name_plural = "End of study"



# Generated by Django 4.2.4 on 2023-08-28 21:26

import _socket
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.encrypted_text_field
import django_revision.revision_field
import edc_model.validators.phone
import edc_model_fields.fields.initials_field
import edc_model_fields.fields.other_charfield
import edc_utils
import edc_utils.date
import simple_history.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_lists", "0008_medicationshortagereasons"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("intecomm_screening", "0059_subjectscreeningug_historicalsubjectscreeningug"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalPatientLogUg",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "legal_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure full name consists of letters only in upper case separated by single spaces",
                                regex="^(([A-Z]+ )*[A-Z]+)?$",
                            )
                        ],
                        verbose_name="Full name",
                    ),
                ),
                (
                    "familiar_name",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Should be a name. Do NOT use MR, MRS, MISS, SIR, MADAM and other such titles. (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure name consists of letters only in upper case separated by single spaces",
                                regex="^(([A-Z]+ )*[A-Z]+)?$",
                            )
                        ],
                        verbose_name="By what NAME should we refer to you? (if we speak to you)",
                    ),
                ),
                (
                    "filing_identifier",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        help_text="Auto-populated when form is saved. <BR>This is a sequential-like identifier to label this patient's paper file",
                        max_length=36,
                        verbose_name="Filing number",
                    ),
                ),
                (
                    "patient_log_identifier",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        help_text="Auto-populated when form is saved. <BR>You may prefer to use the FILING NUMBER. <BR>This identifier is replaced by the 'screening identifier' if the patient screens for the INTECOMM trial",
                        max_length=36,
                        verbose_name="Patient Log Reference",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        db_index=True,
                        default=edc_utils.get_uuid,
                        help_text="Auto populated when screening form is complete",
                        max_length=36,
                    ),
                ),
                (
                    "screening_datetime",
                    models.DateTimeField(
                        blank=True,
                        help_text="Auto populated when screening form is complete",
                        null=True,
                    ),
                ),
                (
                    "subject_identifier",
                    models.CharField(
                        db_index=True,
                        default=edc_utils.get_uuid,
                        help_text="Auto populated when consent form is complete",
                        max_length=36,
                    ),
                ),
                (
                    "consent_datetime",
                    models.DateTimeField(
                        blank=True,
                        help_text="Auto populated when consent form is complete",
                        null=True,
                    ),
                ),
                (
                    "group_identifier",
                    models.CharField(
                        blank=True,
                        help_text="Auto populated when group is randomized",
                        max_length=25,
                        null=True,
                    ),
                ),
                (
                    "initials",
                    edc_model_fields.fields.initials_field.InitialsField(
                        help_text="Type 2-3 letters, all in uppercase and no spaces",
                        max_length=3,
                        verbose_name="Initials",
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(18),
                            django.core.validators.MaxValueValidator(110),
                        ],
                        verbose_name="Age",
                    ),
                ),
                ("report_datetime", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                (
                    "hospital_identifier",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        db_index=True,
                        help_text="Must be unique (Encryption: RSA local)",
                        max_length=71,
                        verbose_name="Hospital identifier",
                    ),
                ),
                (
                    "last_4_hospital_identifier",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        editable=False,
                        help_text="auto-populated (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Last 4 digits of hospital_identifier",
                    ),
                ),
                (
                    "contact_number",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="If unknown, type 'UNKNOWN' (Encryption: RSA local)",
                        max_length=71,
                        validators=[edc_model.validators.phone.phone_number],
                    ),
                ),
                (
                    "alt_contact_number",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_model.validators.phone.phone_number],
                    ),
                ),
                (
                    "last_4_contact_number",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        editable=False,
                        help_text="auto-populated (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        verbose_name="Last 4 digits of contact number",
                    ),
                ),
                (
                    "may_contact",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                        default="tbd",
                        max_length=15,
                        verbose_name="Has the patient agreed to be contacted prior to consent?",
                    ),
                ),
                (
                    "location_description",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text="Street, landmarks near home, etc (Encryption: AES local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "stable",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                        default="tbd",
                        help_text="Refer to the SOP for the definition of 'stable'.",
                        max_length=15,
                        verbose_name="Do the facility health care staff consider the patient stable in care",
                    ),
                ),
                (
                    "last_appt_date",
                    models.DateField(
                        blank=True,
                        help_text="If known, the last appointment may help to estimate the next appointment or the expected frequency of routine appointments",
                        null=True,
                        verbose_name="When was the patient last seen at this health facility",
                    ),
                ),
                (
                    "next_appt_date",
                    models.DateField(
                        blank=True,
                        help_text="If known, this date will help prioritize efforts to contact the patient",
                        null=True,
                        verbose_name="Next scheduled routine appointment at this health facility",
                    ),
                ),
                (
                    "first_health_talk",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                        default="tbd",
                        max_length=15,
                        verbose_name="Attended general health talk",
                    ),
                ),
                (
                    "first_health_talk_date",
                    models.DateField(blank=True, null=True, verbose_name="General talk date"),
                ),
                (
                    "second_health_talk",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                        default="tbd",
                        max_length=15,
                        verbose_name="Attended sensitisation session",
                    ),
                ),
                (
                    "second_health_talk_date",
                    models.DateField(
                        blank=True, null=True, verbose_name="Sensitisation session date"
                    ),
                ),
                (
                    "call_attempts",
                    models.IntegerField(blank=True, default=0, help_text="auto-updated"),
                ),
                (
                    "willing_to_screen",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("tbd", "To be determined")],
                        default="tbd",
                        max_length=15,
                        verbose_name="Has the patient agreed to be screened for the INTECOMM study",
                    ),
                ),
                (
                    "screening_refusal_reason_other",
                    edc_model_fields.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                ("printed", models.BooleanField(default=False)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "screening_refusal_reason",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_lists.screeningrefusalreasons",
                        verbose_name="Reason subject unwilling to screen",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        help_text="This question is asked to confirm you are logged in to the correct site.",
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="intecomm_screening.site",
                        verbose_name="Which site is this?",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Patient Log (Uganda)",
                "verbose_name_plural": "historical Patient Log (Uganda)",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from dateutil.relativedelta import relativedelta
from django.db import models
from django_pandas.managers import DataFrameManager
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel
from edc_qareports.model_mixins import QaReportModelMixin


class Diagnoses(UniqueSubjectIdentifierFieldMixin, QaReportModelMixin, BaseUuidModel):
    """A read-only table with details of each participant's diagnoses.

    Run `python manage.py update_conditions` to populate.
    """

    report_model = models.CharField(max_length=50, default="meta_reports.diagnoses")

    baseline_date = models.DateField(null=True)

    hiv = models.BooleanField(default=False)
    htn = models.BooleanField(default=False)
    dm = models.BooleanField(default=False)

    hiv_dx_date = models.DateField(null=True)
    htn_dx_date = models.DateField(null=True)
    dm_dx_date = models.DateField(null=True)

    hiv_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )
    dm_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )
    htn_dx_days = models.IntegerField(
        default=None, null=True, help_text="Number of days relative to baseline"
    )

    baseline_182d = models.IntegerField(
        default=0,
        help_text="Number of diagnoses that qualify as baseline (182d before baseline)",
    )

    baseline_6m = models.IntegerField(
        default=0,
        help_text="Number of diagnoses that qualify as baseline (6m before baseline)",
    )

    vl_baseline_value = models.IntegerField(null=True)

    vl_baseline_date = models.DateField(null=True)

    vl_endline_value = models.IntegerField(null=True)

    vl_endline_date = models.DateField(null=True)

    comment = models.TextField(null=True)

    objects = DataFrameManager()

    def save(self, *args, **kwargs):
        self.baseline_6m = 0
        if self.hiv and self.hiv_dx_date + relativedelta(months=6) <= self.baseline_date:
            self.baseline_6m += 1
        if self.htn and self.htn_dx_date + relativedelta(months=6) <= self.baseline_date:
            self.baseline_6m += 1
        if self.dm and self.dm_dx_date + relativedelta(months=6) <= self.baseline_date:
            self.baseline_6m += 1
        cuttoff = -182
        hiv = 1 if self.hiv and self.hiv_dx_days < cuttoff else 0
        htn = 1 if self.htn and self.htn_dx_days < cuttoff else 0
        dm = 1 if self.dm and self.dm_dx_days < cuttoff else 0
        self.baseline_182d = hiv + htn + dm
        super().save(*args, **kwargs)

    class Meta(UniqueSubjectIdentifierFieldMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Current Conditions"
        verbose_name_plural = "Current Conditions"



# Generated by Django 4.2.3 on 2023-08-03 16:18

from django.db import migrations
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0087_alter_healtheconomicspatient_pat_citizen_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicspatient",
            name="pat_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If OTHER type of employment, specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicspatient",
            name="pat_employment_type_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=100,
                null=True,
                verbose_name="If OTHER type of employment, specify ...",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-10 17:30

import django.core.validators
import edc_model_fields.fields.custom_django_fields
import edc_model_fields.fields.other_charfield
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_subject",
            "0153_rename_visit_duration_careseekinga_care_visit_duration_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="careseekinga",
            name="missed_activities_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalcareseekinga",
            name="missed_activities_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                metadata="FMEDOTHER1",
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="med_not_collected_reason",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("meds_at_home", "Already had the medicines at home"),
                    ("unavailable", "Medicines were not available"),
                    ("expensive", "Medicines were too expensive"),
                    ("home_remedies", "Home remedies are better"),
                    ("unimportant", "Did not think it was important to get these medicines"),
                    ("too_busy", "Did not have the time to collect or buy medicines"),
                    ("problematic", "Taking medicines caused problems"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FMED1",
                verbose_name="Why were these medicines not received/collected?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekinga",
            name="tests_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="in local currency",
                metadata="FTESTCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much did you spend on these tests?",
            ),
        ),
        migrations.AlterField(
            model_name="careseekingb",
            name="med_not_collected_reason",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("meds_at_home", "Already had the medicines at home"),
                    ("unavailable", "Medicines were not available"),
                    ("expensive", "Medicines were too expensive"),
                    ("home_remedies", "Home remedies are better"),
                    ("unimportant", "Did not think it was important to get these medicines"),
                    ("too_busy", "Did not have the time to collect or buy medicines"),
                    ("problematic", "Taking medicines caused problems"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                metadata="FOUTMEDNO1",
                verbose_name="Why were these medicines not received/collected?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="med_not_collected_reason",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("meds_at_home", "Already had the medicines at home"),
                    ("unavailable", "Medicines were not available"),
                    ("expensive", "Medicines were too expensive"),
                    ("home_remedies", "Home remedies are better"),
                    ("unimportant", "Did not think it was important to get these medicines"),
                    ("too_busy", "Did not have the time to collect or buy medicines"),
                    ("problematic", "Taking medicines caused problems"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                metadata="FMED1",
                verbose_name="Why were these medicines not received/collected?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekinga",
            name="tests_cost",
            field=edc_model_fields.fields.custom_django_fields.IntegerField2(
                blank=True,
                help_text="in local currency",
                metadata="FTESTCOST1",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="How much did you spend on these tests?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcareseekingb",
            name="med_not_collected_reason",
            field=edc_model_fields.fields.custom_django_fields.CharField2(
                choices=[
                    ("meds_at_home", "Already had the medicines at home"),
                    ("unavailable", "Medicines were not available"),
                    ("expensive", "Medicines were too expensive"),
                    ("home_remedies", "Home remedies are better"),
                    ("unimportant", "Did not think it was important to get these medicines"),
                    ("too_busy", "Did not have the time to collect or buy medicines"),
                    ("problematic", "Taking medicines caused problems"),
                    ("OTHER", "Other, please specify ..."),
                    ("N/A", "Not applicable"),
                ],
                max_length=25,
                metadata="FOUTMEDNO1",
                verbose_name="Why were these medicines not received/collected?",
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-04-03 20:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0018_remove_historicalhivinitialreview_vl_date_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalhtninitialreview",
            name="med_start_date_estimated",
        ),
        migrations.RemoveField(
            model_name="htninitialreview",
            name="med_start_date_estimated",
        ),
        migrations.AddField(
            model_name="historicalhtninitialreview",
            name="med_start_date",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="If the patient is taking medicines for hypertension, on which date did they start taking these?",
            ),
        ),
        migrations.AddField(
            model_name="htninitialreview",
            name="med_start_date",
            field=models.DateField(
                blank=True,
                null=True,
                verbose_name="If the patient is taking medicines for hypertension, on which date did they start taking these?",
            ),
        ),
    ]



from django.contrib import admin
from django.urls.conf import include, path
from django.views.generic.base import RedirectView
from edc_action_item.admin_site import edc_action_item_admin
from edc_appointment.admin_site import edc_appointment_admin
from edc_consent.admin_site import edc_consent_admin
from edc_dashboard.views import AdministrationView
from edc_locator.admin_site import edc_locator_admin
from meta_ae.admin_site import meta_ae_admin
from meta_prn.admin_site import meta_prn_admin
from meta_screening.admin_site import meta_screening_admin
from meta_subject.admin_site import meta_subject_admin

from intecomm_consent.admin_site import intecomm_consent_admin

from .admin import meta_test_admin

urlpatterns = [
    path("accounts/", include("edc_auth.urls")),
    path("admin/", include("edc_auth.urls")),
    path("admin/", edc_appointment_admin.urls),
    path("admin/", edc_consent_admin.urls),
    path("admin/", meta_test_admin.urls),
    path("admin/", meta_ae_admin.urls),
    path("admin/", meta_prn_admin.urls),
    path("admin/", edc_locator_admin.urls),
    path("admin/", intecomm_consent_admin.urls),
    path("admin/", meta_subject_admin.urls),
    path("admin/", meta_screening_admin.urls),
    path("admin/", edc_action_item_admin.urls),
    path("admin/", admin.site.urls),
    path("administration/", AdministrationView.as_view(), name="administration_url"),
    path("meta_ae/", include("meta_ae.urls")),
    path("meta_prn/", include("meta_prn.urls")),
    path("meta_screening/", include("meta_screening.urls")),
    path("intecomm_consent/", include("intecomm_consent.urls")),
    path("meta_subject/", include("meta_subject.urls")),
    path("edc_action_item/", include("edc_action_item.urls")),
    path("edc_adverse_event/", include("edc_adverse_event.urls")),
    path("edc_appointment/", include("edc_appointment.urls")),
    path("edc_auth/", include("edc_auth.urls")),
    path("edc_consent/", include("edc_consent.urls")),
    path("edc_dashboard/", include("edc_dashboard.urls")),
    path("edc_data_manager/", include("edc_data_manager.urls")),
    path("edc_device/", include("edc_device.urls")),
    path("edc_lab/", include("edc_lab.urls")),
    path("edc_export/", include("edc_export.urls")),
    path("edc_lab_dashboard/", include("edc_lab_dashboard.urls")),
    path("edc_locator/", include("edc_locator.urls")),
    path("edc_pharmacy_dashboard/", include("edc_pharmacy_dashboard.urls")),
    path("edc_protocol/", include("edc_protocol.urls")),
    path("edc_subject_dashboard/", include("edc_subject_dashboard.urls")),
    path("edc_visit_schedule/", include("edc_visit_schedule.urls")),
    path("subject/", include("intecomm_dashboard.urls")),
    path("i18n/", include("django.conf.urls.i18n")),
    path("", RedirectView.as_view(url="admin/"), name="home_url"),
]
